// SPDX-License-Identifier: BSD-3-Clause

package query_test

import (
	"fmt"
	"net"
	"strconv"
	"strings"
	"testing"

	"git.froth.zone/sam/awl/cli"
	"git.froth.zone/sam/awl/internal/structs"
	"git.froth.zone/sam/awl/query"
	"git.froth.zone/sam/awl/util"

	"github.com/miekg/dns"
	"github.com/stretchr/testify/assert"
)

func TestQuic(t *testing.T) {
	t.Parallel()
	opts := cli.Options{
		QUIC:    true,
		Logger:  util.InitLogger(0),
		Port:    853,
		Request: structs.Request{Server: "dns.adguard.com"},
	}
	testCase := structs.Request{Server: "dns.//./,,adguard.com", Type: dns.TypeA, Name: "git.froth.zone"}
	testCase2 := structs.Request{Server: "dns.adguard.com", Type: dns.TypeA, Name: "git.froth.zone"}
	var testCases []structs.Request
	testCases = append(testCases, testCase)
	testCases = append(testCases, testCase2)
	for i := range testCases {
		switch i {
		case 0:
			resolver, err := query.LoadResolver(testCases[i].Server, opts)
			assert.Nil(t, err)
			// if the domain is not canonical, make it canonical
			if !strings.HasSuffix(testCase.Name, ".") {
				testCases[i].Name = fmt.Sprintf("%s.", testCases[i].Name)
			}
			msg := new(dns.Msg)
			msg.SetQuestion(testCase.Name, testCase.Type)
			msg = msg.SetQuestion(testCase.Name, testCase.Type)
			res, err := resolver.LookUp(msg)
			assert.NotNil(t, err)
			assert.Equal(t, res, structs.Response{})
		case 1:
			resolver, err := query.LoadResolver(testCase2.Server, opts)
			assert.Nil(t, err)
			testCase2.Server = net.JoinHostPort(testCase2.Server, strconv.Itoa(opts.Port))
			// if the domain is not canonical, make it canonical
			if !strings.HasSuffix(testCase2.Name, ".") {
				testCase2.Name = fmt.Sprintf("%s.", testCase2.Name)
			}
			msg := new(dns.Msg)
			msg.SetQuestion(testCase2.Name, testCase2.Type)
			msg = msg.SetQuestion(testCase2.Name, testCase2.Type)
			res, err := resolver.LookUp(msg)
			assert.Nil(t, err)
			assert.NotNil(t, res)
		}

	}

}
